/*
 * $Id: OrderDAOFactory.java,v 1.3 2000/10/09 04:51:19 inder Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.order.ejb;


import javax.naming.NamingException;
import javax.naming.InitialContext;

import java.sql.Connection;
import javax.sql.DataSource;
import java.sql.SQLException;

import com.sun.j2ee.blueprints.customer.util.Debug;
import com.sun.j2ee.blueprints.customer.util.JNDINames;


public class OrderDAOFactory {

    /**
     * This method instantiates a particular subclass implementing
     * the abstract methods based on the information obtained from the
     * Connection object.
     */
    public static OrderDAO getDAO() throws OrderDAOException {

        OrderDAO orderDao = null;
        try {
            InitialContext ic = new InitialContext();
            DataSource ds = (DataSource)
                ic.lookup(JNDINames.ESTORE_DATASOURCE);
            Connection conn = ds.getConnection();
            String DBProductName =
                conn.getMetaData().getDatabaseProductName().trim();
            conn.close();
            if (DBProductName.startsWith("Oracle")) {
                orderDao = new OrderDAOOracle();
                Debug.println("Using Product:" + DBProductName);
            } else if (DBProductName.startsWith("SQL Server")) {
                Debug.println("Using Product:" + DBProductName);
                orderDao = new OrderDAOSybase();
            } else if (DBProductName.startsWith("DBMS:cloudscape")) {
                Debug.println("Using Product:" + DBProductName);
                orderDao = new OrderDAOCS();
            }
        } catch (NamingException ne) {
            throw new OrderDAOException("OrderDAOFactory.getDAO:  NamingException while getting DB connection : \n" + ne.getMessage());
        } catch (SQLException se) {
            throw new OrderDAOException("OrderDAOFactory.getDAO:  SQLException while getting DB connection : \n" + se.getMessage());
        }
        return orderDao;
    }

}
