/*
 * $Id: OrderEJB.java,v 1.15 2000/10/06 15:53:33 lblair Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.order.ejb;

import java.util.Collection;
import java.util.Properties;
import java.rmi.RemoteException;

import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import com.sun.j2ee.blueprints.customer.util.Debug;
import com.sun.j2ee.blueprints.customer.util.EJBUtil;
import com.sun.j2ee.blueprints.customer.util.Calendar;
import com.sun.j2ee.blueprints.customer.util.Address;
import com.sun.j2ee.blueprints.customer.util.CreditCard;
import com.sun.j2ee.blueprints.customer.order.model.LineItem;
import com.sun.j2ee.blueprints.customer.order.model.OrderModel;
import com.sun.j2ee.blueprints.customer.order.ejb.OrderDAOException;
import com.sun.j2ee.blueprints.customer.account.ejb.Account;
import com.sun.j2ee.blueprints.customer.account.ejb.AccountHome;

/**
 * Entity Bean implementation of Order EJB.Uses
 * the Account EJB to get account info associated
 * with an order.
 *
 * @see Account
 */
public class OrderEJB implements EntityBean {

    private Integer orderId;
    private Collection lineItems;
    private String billToFirstName;
    private String billToLastName;
    private String shipToFirstName;
    private String shipToLastName;
    private Address shipToAddr;
    private Address billToAddr;
    private CreditCard chargeCard;
    private String carrier;
    private String userId;
    private Calendar orderDate;
    private String status;
    private double totalPrice;

    private EntityContext context;
    private OrderDAO orderDao;

    public OrderEJB() {}

    public Integer ejbCreate (Collection lineItems, Address shipToAddr,
                              Address billToAddr, String shipToFirstName,
                              String shipToLastName, String billToFirstName,
                              String billToLastName, CreditCard chargeCard,
                              String carrier, String userId,
                              double totalPrice)  throws CreateException {

        // set the instance data
        this.lineItems = lineItems;
        this.shipToAddr = shipToAddr;
        this.billToAddr = billToAddr;
        this.shipToFirstName = shipToFirstName;
        this.shipToLastName = shipToLastName;
        this.billToFirstName = billToFirstName;
        this.billToLastName = billToLastName;

        this.chargeCard = chargeCard;
        this.carrier = carrier;
        this.userId = userId;
        this.orderDate = Calendar.getInstance();
        this.status = Order.PENDING;
        this.totalPrice = totalPrice;

        try {
            OrderDAO dao = getDAO();
            dao.setLineItems(lineItems);
            dao.setShipToAddr(shipToAddr);
            dao.setBillToAddr(billToAddr);
            dao.setShipToFirstName(shipToFirstName);
            dao.setShipToLastName(shipToLastName);
            dao.setBillToFirstName(billToFirstName);
            dao.setBillToLastName(billToLastName);
            dao.setChargeCard(chargeCard);
            dao.setCarrier(carrier);
            dao.setUserId(userId);
            dao.setStatus(Order.PENDING);
            dao.setTotalPrice(totalPrice);

            dao.create();
            orderId = new Integer(dao.getOrderId());
            return orderId;
        } catch (OrderDAOException se) {
            context.setRollbackOnly();
            throw new CreateException (se.getMessage());
        }
    }

    public void ejbRemove() throws RemoveException {

        try {
            OrderDAO dao = getDAO();
            dao.setOrderId((this.orderId).intValue());
            dao.remove();
        } catch (OrderDAOException se) {
            context.setRollbackOnly();
            throw new RemoveException (se.getMessage());
        }
    }

    public void setEntityContext(EntityContext ec) {
        context = ec;
    }

    public void ejbLoad() {
        try{
            OrderDAO dao = getDAO();
            dao.setOrderId((this.orderId).intValue());
            dao.load();
            orderId = new Integer(dao.getOrderId());
            lineItems = dao.getLineItems();
            shipToAddr = dao.getShipToAddr();
            billToAddr = dao.getBillToAddr();
            shipToFirstName = dao.getShipToFirstName();
            shipToLastName = dao.getShipToLastName();
            billToFirstName = dao.getBillToFirstName();
            billToLastName = dao.getBillToLastName();
            chargeCard = dao.getChargeCard();
            carrier = dao.getCarrier();
            userId = dao.getUserId();
            orderDate = dao.getOrderDate();
            status = dao.getStatus();
            totalPrice = dao.getTotalPrice();
        } catch (OrderDAOException se) {
            throw new EJBException (se.getMessage());
        }
    }

    public void ejbStore() {
        try{
            OrderDAO dao = getDAO();

            dao.setOrderId((this.orderId).intValue());
            dao.setLineItems(lineItems);

            dao.setShipToAddr(shipToAddr);
            dao.setBillToAddr(billToAddr);
            dao.setShipToFirstName(shipToFirstName);
            dao.setShipToLastName(shipToLastName);
            dao.setBillToFirstName(billToFirstName);
            dao.setBillToLastName(billToLastName);

            dao.setChargeCard(chargeCard);
            dao.setCarrier(carrier);
            dao.setUserId(userId);
            dao.setOrderDate(orderDate);
            dao.setStatus(status);
            dao.setTotalPrice(totalPrice);

            dao.store();
        } catch (OrderDAOException se) {
            throw new EJBException (se.getMessage());
        }
    }

    /**
     * @param  key is the orderID for a particular order
     */
    public Integer ejbFindByPrimaryKey(Integer key) throws FinderException {
        try{
            OrderDAO dao = getDAO();
            dao.setOrderId(key.intValue());
            return(dao.findByPrimaryKey());
        } catch (OrderDAOException se) {
            throw new FinderException ("SQL Exception in find by primary key");
        }
    }

    /**
     * @return  a Collection of references to all the order EJBs
     *          for the userId.
     */
    public Collection ejbFindUserOrders( String userId )
        throws FinderException {

        try{
            OrderDAO dao = getDAO();
            dao.setUserId(userId);
            return(dao.findUserOrders());
        } catch (OrderDAOException se) {
            throw new FinderException
                ("SQL Exception in finding collection of primary keys");
        }
    }

    public void unsetEntityContext() {}

    public void ejbActivate() {
      this.orderId = (Integer)context.getPrimaryKey();
    }

    public void ejbPassivate() {
      this.orderId  = null;
      this.orderDao = null;
    }

    public void ejbPostCreate(Collection lineItems, Address shipToAddr,
                              Address billToAddr, String shipToFirstName,
                              String shipToLastName, String billToFirstName,
                              String billToLastName, CreditCard chargeCard,
                              String carrier, String userId,double totalPrice)    {}

    /**
     * @return  the OrderModel containing the details
     *           of an order
     */
    public OrderModel getDetails() {
        return(new OrderModel(orderId.intValue(),lineItems, shipToAddr, billToAddr,
                                shipToFirstName, shipToLastName,
                                billToFirstName, billToLastName,
                                chargeCard, carrier, userId,
                                orderDate, status, totalPrice));
    }

    /**
     * @return  the Account EJB associated with this order
     */
    public Account getAccount() {
        try {
            AccountHome acctHome = EJBUtil.getAccountHome();
            return acctHome.findByPrimaryKey(userId);
        } catch (FinderException fe) {
            // Account mysterious disappeared..
            Debug.print(fe);
            throw new EJBException(fe);
        } catch (NamingException ne) {
            // Account mysterious disappeared..
            Debug.print(ne);
            throw new EJBException(ne);
        } catch (RemoteException re) {
            Debug.print(re);
            throw new EJBException(re);
        }
    }

    private OrderDAO getDAO() throws OrderDAOException {
        if(orderDao == null) {
            orderDao = OrderDAOFactory.getDAO();
        }
        return orderDao;
    }

}
